package net.yisu.august.Nativo;

import java.util.Map;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
public class Basic {

	private static final Logger logger = LoggerFactory.getLogger(Basic.class);

	@RequestMapping(value = "${v1API}", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<?> indexGetWithoutSlash() {
		logger.info("the endpoint GET / without the slash was reached");

		JSONObject output = new JSONObject();
		output.put("name", "hello there.");
		return ResponseEntity.status(HttpStatus.OK).body( output.toString() );

	}

	@RequestMapping(value = "${v1API}/", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<?> indexGet() {
		logger.info("the endpoint GET / was reached");

		JSONObject output = new JSONObject();
		output.put("name", "hi there.");
		return ResponseEntity.status(HttpStatus.OK).body( output.toString() );

	}

	@RequestMapping(value = "${v1API}/db", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<?> db() {

		logger.info("the endpoint GET /db was reached");

		JSONObject output = new JSONObject();

		try {

			JdbcTemplate jdbcTemplate = new JdbcTemplate( Utilities.getMySQLDataSourceConnectionPoolException() );

			Map<String, Object> map = jdbcTemplate.queryForMap("SELECT version() AS version");
			logger.info("-- MySQL Version: " + map.get("version") + " --");

			output.put("version", (String) map.get("version"));

		} catch (EmptyResultDataAccessException e) {
			logger.info(e.getLocalizedMessage());
			output.put("EmptyResultDataAccessException", e.getLocalizedMessage() );
		} catch (DataAccessException e) {
			logger.info(e.getLocalizedMessage());
			output.put("DataAccessException", e.getLocalizedMessage() );
		} catch (Exception e) {
			logger.info(e.getLocalizedMessage());
			output.put("Exception", e.getLocalizedMessage() );
		}

		return ResponseEntity.status(HttpStatus.OK).body( output.toString() );

	}
	
}
