package net.yisu.august.Nativo;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.nativex.hint.NativeHint;
import org.springframework.nativex.hint.TypeHint;
import org.springframework.scheduling.annotation.EnableAsync;

import com.mysql.cj.PerConnectionLRUFactory;

@NativeHint(types = {
		@TypeHint(types = {
				PerConnectionLRUFactory.class
		})
})

@SpringBootApplication
@EnableAsync
public class NativoApplication {

	private static final Logger logger = LoggerFactory.getLogger(NativoApplication.class);

	public static void main(String[] args) {

		SpringApplication.run(NativoApplication.class, args);

		logger.info("-- Application Started (Spring Boot v2) --");

		String javaVersion = System.getProperty("java.version");
		logger.info("-- Java Version: " + javaVersion + " --");

		JdbcTemplate jdbcTemplate = new JdbcTemplate( Utilities.getMySQLDataSourceConnectionPool() );

		try {

			Map<String, Object> map = jdbcTemplate.queryForMap("SELECT version() AS version");
			logger.info("-- MySQL Version: " + map.get("version") + " --");

		} catch (EmptyResultDataAccessException e) {
			logger.info(e.getMessage());
		} catch (DataAccessException e) {
			logger.info(e.getMessage());
		}

	}

}