package net.yisu.august.Nativo;

import java.security.MessageDigest;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.servlet.http.HttpServletRequest;
import javax.sql.DataSource;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.datasource.DriverManagerDataSource;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;


public class Utilities implements APIsErrorCodes {

	public static final Logger logger = LoggerFactory.getLogger(Utilities.class);

	public static HikariDataSource ds = null;

	public static String sha256(String base) {

		try {
			MessageDigest digest = MessageDigest.getInstance("SHA-256");
			byte[] hash = digest.digest(base.getBytes("UTF-8"));
			StringBuffer hexString = new StringBuffer();

			for (int i = 0; i < hash.length; i++) {

				String hex = Integer.toHexString(0xff & hash[i]);

				if (hex.length() == 1) {
					hexString.append('0');
				}

				hexString.append(hex);
			}

			return hexString.toString();
		} catch (Exception ex) {
			return "INVALID";
//			throw new RuntimeException(ex);
		}

	}

	public static String throwError(String message) {

		JSONObject item = new JSONObject();
		item.put("status", "error");
		item.put("message", message);

		return item.toString();

	}

	public static String throwEMailError(String message, String email) {

		JSONObject item = new JSONObject();
		item.put("status", "error");
		item.put("message", message);
		item.put("email", email);

		return item.toString();

	}

	public static String successfulResponse(String message) {

		JSONObject item = new JSONObject();
		item.put("status", "successful");
		item.put("message", message);

		return item.toString();

	}



	public static Connection getDBConnection() {

		Connection dbConnection = null;

		String url = Configurator.MYSQL_CONNECTION_STRING;
		String user = Configurator.MYSQL_USERNAME;
		String password = Configurator.MYSQL_PASSWORD;

		try {

			dbConnection = DriverManager.getConnection(url, user, password);
			return dbConnection;

		} catch (SQLException e) {
			logger.info(e.getMessage());
		}

		return dbConnection;

	}

	public static DataSource getMySQLDataSource_DEPRECATED() {

		//	This method was DEPRECATED because I started using the Hikari's Connection Pool

		String driverClassName = Configurator.MYSQL_DRIVER_CLASS_NAME;
		String url = Configurator.MYSQL_CONNECTION_STRING;
		String user = Configurator.MYSQL_USERNAME;
		String password = Configurator.MYSQL_PASSWORD;

		DriverManagerDataSource dataSource = new DriverManagerDataSource();

		dataSource.setDriverClassName( driverClassName );
		dataSource.setUrl( url );
		dataSource.setUsername( user );
		dataSource.setPassword( password );

		return dataSource;
	}

	public static DataSource getMySQLDataSourceConnectionPool() {

/*
		private static final long CONNECTION_TIMEOUT = SECONDS.toMillis(30);
		private static final long VALIDATION_TIMEOUT = SECONDS.toMillis(5);
		private static final long IDLE_TIMEOUT = MINUTES.toMillis(10);
		private static final long MAX_LIFETIME = MINUTES.toMillis(30);
		maxLifetime = MAX_LIFETIME;
		connectionTimeout = CONNECTION_TIMEOUT;
		validationTimeout = VALIDATION_TIMEOUT;
		idleTimeout = IDLE_TIMEOUT;
*/

		HikariConfig config = new HikariConfig();

		String driverClassName = Configurator.MYSQL_DRIVER_CLASS_NAME;
		String url = Configurator.MYSQL_CONNECTION_STRING;
		String user = Configurator.MYSQL_USERNAME;
		String password = Configurator.MYSQL_PASSWORD;

		config.setDriverClassName(driverClassName);
		config.setJdbcUrl(url);
		config.setUsername(user);
		config.setPassword(password);
		config.setMaximumPoolSize(3);

		config.setConnectionTimeout( 10000 );		//	 10 seconds
		config.setMaxLifetime( 600000 );			//	600 seconds = 10 minutes


//		###		idleTimeout has been set but has no effect because the pool is operating as a fixed size pool.
//		###		config.setIdleTimeout(  60000 );		//	 60 seconds = 1 minute

		config.addDataSourceProperty("cachePrepStmts", "true");
		config.addDataSourceProperty("prepStmtCacheSize", "250");
		config.addDataSourceProperty("prepStmtCacheSqlLimit", "2048");

		if (ds == null) {
			ds = new HikariDataSource(config);
		}

		return ds;

	}

	public static DataSource getMySQLDataSourceConnectionPoolException() throws Exception {

/*
		private static final long CONNECTION_TIMEOUT = SECONDS.toMillis(30);
		private static final long VALIDATION_TIMEOUT = SECONDS.toMillis(5);
		private static final long IDLE_TIMEOUT = MINUTES.toMillis(10);
		private static final long MAX_LIFETIME = MINUTES.toMillis(30);
		maxLifetime = MAX_LIFETIME;
		connectionTimeout = CONNECTION_TIMEOUT;
		validationTimeout = VALIDATION_TIMEOUT;
		idleTimeout = IDLE_TIMEOUT;
*/

		HikariConfig config = new HikariConfig();

		String driverClassName = Configurator.MYSQL_DRIVER_CLASS_NAME;
		String url = Configurator.MYSQL_CONNECTION_STRING;
		String user = Configurator.MYSQL_USERNAME;
		String password = Configurator.MYSQL_PASSWORD;

		config.setDriverClassName(driverClassName);
		config.setJdbcUrl(url);
		config.setUsername(user);
		config.setPassword(password);
		config.setMaximumPoolSize(2);

		config.setConnectionTimeout( 1000 );		//	 1 seconds
		config.setMaxLifetime( 600000 );			//	600 seconds = 10 minutes


//		###		idleTimeout has been set but has no effect because the pool is operating as a fixed size pool.
//		###		config.setIdleTimeout(  60000 );		//	 60 seconds = 1 minute

		config.addDataSourceProperty("cachePrepStmts", "true");
		config.addDataSourceProperty("prepStmtCacheSize", "250");
		config.addDataSourceProperty("prepStmtCacheSqlLimit", "2048");

		if (ds == null) {
			ds = new HikariDataSource(config);
		}

		return ds;

	}

	public static String parseTimestampToString(Timestamp myTimestamp) {

		if (myTimestamp == null) {
			return "";
		}

		Date date = new Date( myTimestamp.getTime() );

		SimpleDateFormat dateFormatGmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		dateFormatGmt.setTimeZone(TimeZone.getTimeZone("UTC"));

		return dateFormatGmt.format(date);

	}

}
